#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Dieses Dokument ist in der Zusammenarbeit von Informatikreferendaren und
# ehemaligen Informatikreferendaren der Studienseminare (heute ZfsL) Arnsberg,
# Hamm und Solingen entstanden.
#
# Das Dokument steht unter der Lizenz: Creative Commons by-nc-sa Version 4.0
# http://creativecommons.org/licenses/by-nc-sa/4.0/deed.de
#
# Nach dieser Lizenz darf das Dokument beliebig kopiert und bearbeitet werden,
# sofern das Folgeprodukt wiederum unter gleichen Lizenzbedingungen vertrieben
# und auf die ursprünglichen Urheber verwiesen wird.
# Eine kommerzielle Nutzung ist ausdrücklich ausgeschlossen.
#
# Die Namensnennung durch einen Verweis und die Lizenzangabe der ursprünglichen
# Urheber auf den Materialien für Schülerinnen und Schüler ist erforderlich.
#
# Die Sammlung der Dokumente steht unter
# http://ddi.uni-wuppertal.de/material/materialsammlung/
# zur Verfügung.
#
# Weitere Lizenz: GPL V3.0
# Siehe http://www.gnu.org/licenses/gpl-3.0.html
#
# $Id: kino.py 1225 2015-05-26 08:00:19Z humbert $

class Kino:
	"""Klasse Kino"""
	def __init__(self, neuName, neuOrt):
		"""Konstruktor -- Argumente: neuName, neuOrt """
		self.name= neuName
		self.ort= neuOrt
		self.saal4= Saal("Saal 4", 334)
		self.saal6= Saal("Saal 6", 128)
		self.kasse= Kasse()
	def gibName(self):
		return self.name
	def gibOrt(self):
		return self.ort
	def gibKasse(self):
		return self.kasse
	def gibSaal4(self):
		return self.saal4
	def gibSaal6(self):
		return self.saal6

class Kinobesucher:
	"""Klasse Kinobesucher"""
	def __init__(self, neuName, neuKino):
		"""Konstruktor -- Argumente: neuName, neuKino """
		self.name= neuName
		self.ticket= None
		self.kino=  neuKino
	def gibName(self):
		return self.name
	def gibTicket(self):
		return self.ticket
	def ticketAbholen(self, fuerFilm):
		"""Argument: fuerFilm """
		self.ticket = self.kino.gibKasse().ticketAusgeben(fuerFilm)
	def ticketUmtauschen(self, neuerFilm):
		"""Argument: neuerFilm """
		self.ticket = self.kino.gibKasse().ticketUmtauschen(self.ticket, neuerFilm)

class Film:
	"""Klasse Film"""
	def __init__(self, neuName, neuStartzeit):
		"""Konstruktor -- Argumente: neuName, neuStartzeit """
		self.name= neuName
		self.startzeit= neuStartzeit
	def gibName(self):
		return self.name
	def gibStartzeit(self):
		return self.startzeit
	def gibSaal(self):
		return self.saal
	def setzeSaal(self, neuSaal):
		self.saal= neuSaal
		
class Saal:
	"""Klasse Saal"""
	def __init__(self, neuName, neuSitzplaetze):
		"""Konstruktor -- Argumente: neuName, neuSitzplaetze """
		self.name= neuName
		self.sitzplaetze= neuSitzplaetze
		self.freiePlaetze= neuSitzplaetze
		self.film= None
	def gibFreiePlaetze(self):
		return self.freiePlaetze
	def erhoeheFreiePlaetzeUm(self, anzahl):
		self.freiePlaetze= self.freiePlaetze + anzahl
	def gibName(self):
		return self.name
	def setzeName(self, neuName):
		self.name= neuName
	def gibFilm(self):
		return self.film
	def setzeFilm(self, derFilm):
		"""Argument: derFilm (Film, der in diesem Saal gezeigt wird)"""
		self.film= derFilm

	def leeren(self):
		self.freiePlaetze= self.sitzplaetze

	def zeigeAusgabeFuerAnzeigetafel(self):
		print (self.name + ": gespielt wird " + self.gibFilm().gibName() + 
		  " (" + str(self.freiePlaetze) + " von " + str(self.sitzplaetze) + 
		  " Plätzen frei).")

class Kasse:
	"""Klasse Kasse"""
	def ticketAusgeben(self, derFilm):
		"""Argument: derFilm (Film, für den ein Ticket gewünscht wird)"""
		freiePlaetze= derFilm.gibSaal().gibFreiePlaetze()
		if freiePlaetze > 0:
			derFilm.gibSaal().erhoeheFreiePlaetzeUm(-1)
			return Ticket(derFilm)

	def ticketUmtauschen(self, altesTicket, denFilm):
		"""Argumente: altesTicket, denFilm (vernichtet altesTicket 
		   und gibt ein neues Ticket für denFilm aus)"""
		altesTicket.gibFilm().gibSaal().erhoeheFreiePlaetzeUm(1)
		altesTicket= None
		return self.ticketAusgeben(denFilm)

class Ticket:
	"""Klasse Ticket"""
	def __init__(self, derFilm):
		"""Konstruktor -- Argument: derFilm (erzeugt 
		   ein Ticket für den Film derFilm)"""
		self.film= derFilm
		print ("Es wurde ein Ticket ausgestellt für den Film " +
		   self.film.gibName() + ".")
	def __del__(self):
		print ("Ein Ticket für den Film " + self.film.gibName() +
		   " wurde zurückgenommen.")
	def gibFilm(self):
		return self.film
	def gibAufschrift(self):
		return "Ticket für den Film " + self.film.gibName() + " um " \
		  + self.film.gibStartzeit() + " in " + self.film.gibSaal().gibName()


if __name__ == "__main__":
	cinedome = Kino("CineDome", "Hinterschaffetten")
	fdk3 = Film("Fluch der Karibik 3", "19:30")
	mbmf = Film("Mr. Bean macht Ferien", "19.45")

	fdk3.setzeSaal(cinedome.gibSaal4())
	mbmf.setzeSaal(cinedome.gibSaal6())
	cinedome.gibSaal4().setzeFilm(fdk3)
	cinedome.gibSaal6().setzeFilm(mbmf)

	jannik = Kinobesucher("Jannik", cinedome)
	laura = Kinobesucher("Laura", cinedome)
	maex = Kinobesucher("Max", cinedome)
	
	cinedome.gibSaal4().zeigeAusgabeFuerAnzeigetafel()
	cinedome.gibSaal6().zeigeAusgabeFuerAnzeigetafel()

	jannik.ticketAbholen(fdk3)
	laura.ticketAbholen(fdk3)
	
	cinedome.gibSaal4().zeigeAusgabeFuerAnzeigetafel()
	cinedome.gibSaal6().zeigeAusgabeFuerAnzeigetafel()
	
	maex.ticketAbholen(mbmf)
	
	cinedome.gibSaal4().zeigeAusgabeFuerAnzeigetafel()
	cinedome.gibSaal6().zeigeAusgabeFuerAnzeigetafel()

	jannik.ticketUmtauschen(mbmf)
	laura.ticketUmtauschen(mbmf)

	cinedome.gibSaal4().zeigeAusgabeFuerAnzeigetafel()
	cinedome.gibSaal6().zeigeAusgabeFuerAnzeigetafel()
	
