#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Dieses Dokument ist in der Zusammenarbeit von Informatikreferendaren und
# ehemaligen Informatikreferendaren der Studienseminare (heute ZfsL) Arnsberg,
# Hamm und Solingen entstanden.
#
# Das Dokument steht unter der Lizenz: Creative Commons by-nc-sa Version 4.0
# http://creativecommons.org/licenses/by-nc-sa/4.0/deed.de
#
# Nach dieser Lizenz darf das Dokument beliebig kopiert und bearbeitet werden,
# sofern das Folgeprodukt wiederum unter gleichen Lizenzbedingungen vertrieben
# und auf die ursprünglichen Urheber verwiesen wird.
# Eine kommerzielle Nutzung ist ausdrücklich ausgeschlossen.
#
# Die Namensnennung durch einen Verweis und die Lizenzangabe der ursprünglichen
# Urheber auf den Materialien für Schülerinnen und Schüler ist erforderlich.
#
# Die Sammlung der Dokumente steht unter
# http://ddi.uni-wuppertal.de/material/materialsammlung/
# zur Verfügung.
#
# Weitere Lizenz: GPL V3.0
# Siehe http://www.gnu.org/licenses/gpl-3.0.html
#
# $Id: playlisten.py 582 2014-04-19 19:19:24Z pieper $

################################################################################
# Musterloesung: Music-Center Jhg 11.
# Version 1
# Einschraenkungen:
# -----------------
# - Erstellung von (Musik)Playlisten fuer Gruppen von Personen
# - Kriterium: Playlisten enthalten die Stuecke die alle Personen gerne hören 
# - es gibt nur eine Gruppe mit genau einer Playliste
# - Methode abspielen() soll Playliste im einfachen M3U-Format in das
#   aktuelle Arbeitsverzeichnis speichern, kann von vielen Mediaplayern 
#   abgespielt werden.
# Moegliche Erweiterungen bzw Nachfolgeversionen:
# -----------------------------------------------
# - verschiedenen/mehrere Kriterien zur Erstellung der Playlisten
# - mehrere Gruppen mit mehreren Playlisten zulassen
################################################################################


class Person(object):
    def __init__(self,nameNeu):
        print ('Objekt der Klasse Person erstellt')
        self.name = nameNeu         # name wird bei Objekterstellung mitgegeben
        self.lieblingsStuecke = []  # Liste lieblingsStuecke ist zunaechst leer
        
    def stueckEntfernen(self,stueck):
        self.lieblingsStuecke.remove(stueck)

    def stueckHinzu(self,stueck):
        self.lieblingsStuecke.append(stueck)

    def gibLieblingsStuecke(self):
        return self.lieblingsStuecke

    def gibName(self):              # nur zum Testen
        return self.name


class Stueck(object):
    def __init__(self,interpretNeu,titelNeu,dateiNeu):
        print ('Objekt der Klasse Stueck erstellt')
        self.interpret = interpretNeu     # interpret wird bei Objekterstellung mitgegeben
        self.titel = titelNeu             # titel wird bei Objekterstellung mitgegeben
        self.datei = dateiNeu             # datei wird bei Objekterstellung mitgegeben

    def abspielen(self,datei):        
        print ('spiele: ' + self.interpret + ':' + self.titel)
        datei.write(self.datei)           # Dateiname in datei für Playlisten schreiben
        datei.write('\n')
            

class Playliste(object):
    def __init__(self):
        print ('Objekt der Klasse Playliste erstellt')
        self.stuecke = []                # Liste stuecke ist zunaechst leer
        self.dateiname = 'playliste.m3u'
                                         
    def stueckEntfernen(self,stueck):
        self.stuecke.remove(stueck)

    def stueckHinzu(self,stueck):
        self.stuecke.append(stueck)

    def abspielen(self):
        print ('Abspielen der Playliste')
        datei = open(self.dateiname,'w')
        for s in self.stuecke:
            s.abspielen(datei)
        datei.close()
        print ('Ende der Playliste')
            
    def setzteStuecke(self,listeNeu):
        self.stuecke = listeNeu
        

class Gruppe(object):
    def __init__(self):
        print ('Objekt der Klasse Gruppe erstellt')
        self.personen = []               # Liste personen ist zunaechst leer
        self.playliste = Playliste()     # Die Gruppe hat die Playliste

    def gibPersonen(self):
        return self.personen

    def personEntfernen(self,person):
        self.personen.remove(person)

    def personHinzu(self,person):
        self.personen.append(person)

    def abspielen(self):
        self.playliste.abspielen()

    def erstellePlayliste(self):
        print ('erstellePlayliste')
        liste = []                       # wie erklärt man liste? Oder besser liste als Hilfsattribut?
        liste = self.gleicheStuecke(self.personen)
        # Erweiterung der Erzeugungskriterien koennte wie folgt aussehen:
        # if liste==[]: 
        #     liste = self.Zufall(self.personen)
        # if liste==[]: 
        #     liste = self.Gemischt(self.personen)
        # ...
        self.playliste.setzteStuecke(liste)

    def gleicheStuecke(self, personen):
        if personen == []:
            print ('keine Personen in Gruppe')
            return []
        else:
            schnitt = personen[0].gibLieblingsStuecke()
            for p in personen:
                schnitt = self.findeGleiche(schnitt,p.gibLieblingsStuecke())
            return schnitt

    def findeGleiche(self,liste1,liste2):
        gleiche = []                     # Liste gleiche enthael1 alle Stuecke die in liste1 und liste2 vorkommen
        for stueck in liste1:            # fuer jedes stueck in liste1 
            if stueck in liste2:         # wenn stueck auch in liste2 
                gleiche.append(stueck)   # dann gehoert stueck in die Liste gleiche
        return gleiche
